#include "global.h"
#include "game.h"
#include "title.h"
#include "timer.h"
#include "images.h"

using namespace ol;

static const int TARGET_H = 768;

// Varibles from global.h
int v_w, v_h;
float text_scale_x, text_scale_y;
TextRenderer game;
Bitmap *king, *peon, *gold;
SAMPLE *kinghit, *kingattack, *kingdead, *peonhit, *peonattack, *peondead;

static void init_data(void) {
	game.Load("media/BOLONEWT.TTF", TEXT_X(20), TEXT_Y(20), Rgba::WHITE);
	// Load Music here
	// Until I get around to making them, graphics are NULL
	king = new Bitmap("media/king.png");
	peon = new Bitmap("media/peon.png");
	gold = new Bitmap("media/gold.png");
	images.load();
	kinghit = load_sample("media/kinghit.wav");
	kingattack = load_sample("media/kingattack.wav");
	kingdead = load_sample("media/kingdead.wav");
	peonhit = load_sample("media/peonhit.wav");
	peonattack = load_sample("media/peonattack.wav");
	peondead = load_sample("media/peondead.wav");
	srand(time(NULL));
}

static void clean_up_data(void) {
	// Remove music here
	images.destroy();
	if (king)
		delete king;
	if (peon)
		delete peon;
	if (gold)
		delete gold;
	if (peonhit)
		destroy_sample(peonhit);
	if (peonattack)
		destroy_sample(peonattack);
	if (peondead)
		destroy_sample(peondead);
	if (kinghit)
		destroy_sample(kinghit);
	if (kingattack)
		destroy_sample(kingattack);
	if (kingdead)
		destroy_sample(kingdead);
}

static void setupScreen() {
	int dw, dh, windowed, sw, sh;
	float scale, ratio;
	char *sratio;
	if (get_desktop_resolution(&dw, &dh) == 0) {
		windowed = get_config_int("settings", "windowed", 1);
		scale = get_config_float("settings", "win_scale", 75)/100.0;
		sratio = (char *)get_config_string("settings", "win_ratio", "M");
		ratio = (float)dw/(float)dh;
		v_h = TARGET_H;
		if (windowed) {
			// Override ratio from the desktop res
			switch (sratio[0]) {
			case 'S':
			case 's':
				ratio = 4.0/3.0;
				break;
			case 'W':
			case 'w':
				ratio = 16.0/9.0;
				break;
			}
			if (ratio == (float)dw/(float)dh) {
				sh = (int)(dh*scale);
				sw = (int)(dw*scale);
			}
			else if (ratio > (float)dw/(float)dh) {
				sw = (int)(dw*scale);
				sh = (int)(sw/ratio);
			}
			else {
				sh = (int)(dh*scale);
				sw = (int)(sh/ratio);
			}
			if (!Setup::SetupScreen(sw, sh, false, desktop_color_depth())) {
				allegro_message("Can't set windowed mode with your desktop resolution and scale. Check your drivers, or try another ratio.\n");
				exit(1);
			}
		}
		else {
			if (!Setup::SetupScreen(dw, dh, true)) {
				allegro_message("Can't set fullscreen mode with your desktop resolution. Check your drivers, or try windowed mode.\n");
				exit(1);
			}
			sw = dw;
			sh = dh;
		}
		v_w = (int)(v_h*ratio);
		// Adjust text scaling, because OpenLayer ignores ortho settings below... <grumble grumble grumble>
		
		text_scale_x = ((float)sw/(float)v_w);
		text_scale_y = ((float)sh/(float)v_h);
		
		Settings::SetOrthographicProjection(v_w, v_h);
		OlLog("WindowWidth() = " + ToString(Setup::GetWindowWidth()));
		OlLog("WindowHeight() = " + ToString(Setup::GetWindowHeight()));
		OlLog("Virtual screen = " + ToString(v_w) + "x" + ToString(v_h));
		OlLog("Ratio = " + ToString(ratio));
		OlLog("Text Scale = " + ToString(text_scale_x) + "x" + ToString(text_scale_y));
	}
	else {
		allegro_message("Sorry, but your platform is not supported. If you're on some form of Unix, run inside of X Windows.\n");
		exit(1);
	}
}

int main() {
	MIDI *bgMus;
	srand(time(NULL));
	Setup::SetupProgram(KEYBOARD|TIMER);
	override_config_file("stealingold.ini");
	set_window_title("Stealin Gold");
	Timer::Intialize();
	
	if (install_sound(DIGI_AUTODETECT, MIDI_AUTODETECT, 0) != 0) {
		allegro_message("Error setting up the sound!\n%s\n", allegro_error);
		install_sound(DIGI_NONE, MIDI_NONE, 0);
	}

	setupScreen();
	Settings::StoreMemoryBitmaps(true);
	init_data();

	bgMus = load_midi("media/bjorn__lynne-the_chaos_warrior.mid");
	if (bgMus) {
		play_midi(bgMus, 1);
	}
	while (title_screen())
		play_game();

	destroy_midi(bgMus);
	clean_up_data();
	return 0;
}
END_OF_MAIN()
