#include <math.h>
#include "global.h"
#include "mfobject.h"
#include "images.h"

using namespace ol;

Vec2D strike_pos;
static bool debug = false;

MFObject::MFObject(int type, Vec2D pos, bool gold) : _type(type), _pos(pos) {
	// Reset everything to zero
	state = _score = num_gold = dir = damage = ai_stun = 0;
	hit_played = dead_played = false;
	counter = 0.0;
	hit = -1;
	child = NULL;
	collide = false;
	// type is 0 for king, 1 for peon, 2 for gold 5/6 for weapon strike zone
	switch (type) {
	case 0:
		_health = 20; // 20 hits is all you get
		dir = 1;
		break;
	case 1:
		if (gold) num_gold++; // Peons can start with gold
		_health = 3; // Only when hit without an item does this drop
		break;
	case 2:
	case 4:
		_health = 1;
		break;
	case 5:
	case 6:
		_health = 20;
	}
}

void MFObject::doCollide(MFObject *other) {
	if (_health < 1) return; // This object will be gone at some point
	Vec2D bump( other->_pos );
	bump -= _pos;
	float dist = bump.GetMagnitudeSquared();
	
	if (dist < 40*40) {
		collide = true;
	}
	else {
		collide = false;
	}
		
	if (collide && _type > 1 && _type < 5) {
		// Tis an item.
		if (other->_type == 0) {
			// Item being picked up
			if (_type == 2) {
				other->num_gold++;
				other->_score += 1000;
			}
			_health = 0; // Item gone now
		}
	}
	else if(_type == 1) {
		// tis a peon
		if (other->_type == 0) {
			target_pos = other->_pos;
		}
		else if (collide && other->_type == 5 && !damage) {
			collide_angle = atan2(bump.y, bump.x);
			collide_vec = other->_pos;
			damage = 30;
			if (!child) {
				if (num_gold) {
					num_gold--;
					child = new MFObject(2, _pos);
				}
				else {
					if (_health > 0) _health--;
					if (_health == 0 && !dead_played) {
						dead_played = true;
						play_sample(peondead, 255, 128, 1000, 0);
					}
				}
				if (!dead_played) play_sample(peonhit, 255, 128, 1000, 0);
			}
		}
	}
	else if(_type == 0) {
		// tis the king
		if (collide && other->_type == 6 && !damage) {
			damage = 30;
			if (_health > 0) _health--;
			if (_health == 0 && !dead_played) {
				dead_played = true;
				play_sample(kingdead, 255, 128, 1000, 0);
			}
			if (!dead_played) play_sample(kinghit, 255, 128, 1000, 0);
		}
	}
	else {
		// Not hitting something we respond to
		collide = false;
	}
};

bool MFObject::doLogic(int **map) {
	int map_x, map_y;
	int x, y;
	float fx, fy;
	if (damage > 0) damage--;
	if (hit > 0) hit--;
	if (_health == -1) return false; // King or peon is dead, and animation is done
	switch (_type) {
	case 0: {
		map_x = (int)(_pos.x+32)/64;
		map_y = (int)(_pos.y+32)/64;
		switch (map[map_x][map_y]) {
		case 0:
			// Started the logic frame in the wall? Huh? Bug?
			return false;
		case 1:
			speed *= 0.5;
			break;
		case 2:
			speed *= 0.05;
			break;
		case 3:
			speed *= 0.95;
			break;
		}
		Vec2D force;
		if (state != 2 || state != 4) {
			if (hit <= 1) {
				if (key[KEY_SPACE] && hit == -1) {
					play_sample(kingattack, 255, 128, 1000, 0);
					hit = 21;
					state = 2;
					speed.x = speed.y = 0; // Stop in your tracks
					x = y = 0;
					switch (dir) {
					case 0:
						y = -40;
						break;
					case 1:
						y = 40;
						break;
					case 2:
						x = -40;
						break;
					case 3:
						x = 40;
						break;
					}
					child = new MFObject(5, _pos+Vec2D(x, y));
				}
				else if (!key[KEY_SPACE] && hit == 0) {
					hit = -1;
				}
				if (key[KEY_UP] || key[KEY_W]) {
					force.y -= 1;
					dir = 0;
				}
				if (key[KEY_DOWN] || key[KEY_S]) {
					force.y += 1;
					dir = 1;
				}
				if (key[KEY_LEFT] || key[KEY_A]) {
					force.x -= 1;
					dir = 2;
				}
				if (key[KEY_RIGHT] || key[KEY_D]) {
					force.x += 1;
					dir = 3;
				}
				if (force.x && force.y) force *= .7071;
			}
		}
		
		force *= 2; // Adjust speed here
		if (map[map_x][map_y] == 3) force *= 0.3;
		speed += force;
		if (speed.x > 5.0) speed.x = 5.0;
		if (speed.x < -5.0) speed.x = -5.0;
		if (speed.y > 5.0) speed.y = 5.0;
		if (speed.y < -5.0) speed.y = -5.0;
		_pos = _pos + speed;
		map_x = (int)(_pos.x+32)/64;
		map_y = (int)(_pos.y+32)/64;
		if (map[map_x][map_y] == 0) {
			// Hit a wall, return to the last spot, and set speed to 0
			_pos = _pos - speed;
			speed.x = 0;
			speed.y = 0;
		}
		// Move camera to King
		map_x = (int)(_pos.x + 32 - (v_w/2));
		map_y = (int)(_pos.y + 32 - (v_h/2));
		if (map_x < 0) map_x = 0;
		if (map_x > 3200 - v_w) map_x = 3200 - v_w;
		if (map_y < -30) map_y = -30;
		if (map_y > 3200 - v_h) map_y = 3200 - v_h;
		Transforms::SetPosition(-map_x, -map_y);
		break;
	}
	case 1: {
		float dist;
		if (ai_stun > 0) ai_stun--;
		speed.x = speed.y = 0;
		float angle = atan2(_pos.y - target_pos.y, _pos.x - target_pos.x);
		if (angle >= M_PI/4 && angle <= 3*M_PI/4) {
			dir = 0;
		} else if (angle >= -3*M_PI/4 && angle <= -M_PI/4) {
			dir = 1;
		} else if (angle <= M_PI/4 && angle >= -M_PI/4) {
			dir = 2;
		} else {
			dir = 3;
		}
		if (state != 4  && !ai_stun && !damage) {
			// move to target, the farther away, the fast the peon moves, at most, 7 units
			fx = _pos.x - target_pos.x;
			fy = _pos.y - target_pos.y;
			dist = sqrt(fy*fy + fx*fx);
			if (dist > 0 && dist < 60) {
				// At the target. If the target is the king, attack! Otherwise, we should be focused on getting something else by now
				speed.x = speed.y = 0;
				hit = 21;
				ai_stun = 50;
				state = 2;
				x = y = 0;
				switch (dir) {
				case 0:
					y = -32;
					break;
				case 1:
					y = 32;
					break;
				case 2:
					x = -32;
					break;
				case 3:
					x = 32;
					break;
				}
				child = new MFObject(6, _pos+Vec2D(x, y));
			}
			else if (dist < 192) {
				// near the target, but not really close enough to do anything, move a unit
				if (target_pos.x > _pos.x) speed.x = 1;
				if (target_pos.x < _pos.x) speed.x = -1;
				if (target_pos.y > _pos.y) speed.y = 1;
				if (target_pos.y < _pos.y) speed.y = -1;
			}
			else if (dist < 640) {
				// Far from the target, run then walk as we get closer
				dist -= 164;
				if (target_pos.x > _pos.x) speed.x = dist/64;
				if (target_pos.x < _pos.x) speed.x = -dist/64;
				if (target_pos.y > _pos.y) speed.y = dist/64;
				if (target_pos.y < _pos.y) speed.y = -dist/64;
				// Cap the speed
				if (speed.x > 7) speed.x = 7;
				if (speed.x < -7) speed.x = -7;
				if (speed.y > 7) speed.y = 7;
				if (speed.y < -7) speed.y = -7;
				if (speed.x < 1 && speed.x > 0) speed.x = 1;
				if (speed.x > -1 && speed.x < 0) speed.x = -1;
				if (speed.y < 1 && speed.y > 0) speed.y = 1;
				if (speed.y > -1 && speed.y < 0) speed.y = -1;
			}
			else {
				speed.x = speed.y = 0;
			}
		}
		else {
			speed.x = speed.y = 0;
		}
		
		if (damage > 0 && state != 2) {
			// Override the speed anyway
			if (target_pos.x > _pos.x) speed.x = -5;
			if (target_pos.x < _pos.x) speed.x = 5;
			if (target_pos.y > _pos.y) speed.y = -5;
			if (target_pos.y < _pos.y) speed.y = 5;
		}
		map_x = (int)(_pos.x+32)/64;
		map_y = (int)(_pos.y+32)/64;
		switch (map[map_x][map_y]) {
		case 0:
			// Started the logic frame in the wall? Huh? Bug?
			return false;
		case 1:
			speed *= 0.5;
			break;
		case 2:
			speed *= 0.1;
			break;
		case 3:
			speed *= 0.95;
			break;
		}
		_pos = _pos + speed;
		map_x = (int)(_pos.x+32)/64;
		map_y = (int)(_pos.y+32)/64;
		if (map[map_x][map_y] == 0) {
			// Hit a wall, return to the last spot, and set speed to 0
			_pos = _pos - speed;
			speed.x = 0;
			speed.y = 0;
		}
		break;
	}
	case 2:
	case 3:
	case 4:
		if (_health == 0) return false; // Item picked up, remove it
		break;
	case 5:
	case 6:
		_health--;
		if (_health < 0) return false;
		break;
	default:
		break;
	}
	
	// Handle hitting, attacking, and death here
	if (_type < 2 && _health > 0) {
		switch (state) {
		case 0:
			if (fabs(speed.x) > 0.2 || fabs(speed.y) > 0.2) {
				counter = 0;
				state = 1;
			}
			else if (hit > 0) {
				state = 2;
				counter = 0;
			}
			else if (damage > 0) {
				counter = 0;
				state = 3;
			}
			break;
		case 1:
			if (fabs(speed.x) < 0.2 || fabs(speed.y) < 0.2) {
				counter = 0;
				state = 0;
			}
			else if (damage > 0) {
				state = 3;
				counter = 0;
			}
			break;
		case 2:
			if (hit < 1) {
				counter = 0;
				if (fabs(speed.x) > 0.2 || fabs(speed.y) > 0.2) {
					state = 1;
				}
				else {
					state = 0;
				}
			}
			if (damage > 0) {
				counter = 0;
				state = 3;
			}
			break;
		case 3:
			if (damage < 1) {
				counter = 0;
				if (fabs(speed.x) > 0.1 || fabs(speed.y) > 0.1) {
					state = 1;
				}
				else {
					state = 0;
				}
			}
		}
	}
	else {
		state = 4;
	}
	collide = false;
	return true;
}

void MFObject::draw() {
	counter += FpsCounter::GetDeltaTime();
	switch (_type) {
	case 0:
		if (!images.drawKing(dir, state, _pos, (int)counter)) _health = -1;
		break;
	case 1:
		if (!images.drawPeon(dir, state, _pos, (int)counter)) _health = -1;
		break;
	case 2:
	case 3:
	case 4:
		images.drawItem(_type-2, _pos, (int)counter);
		break;
	case 5:
	case 6:
		Circle(Vec2D(_pos.x+32, _pos.y+32), 32).Draw(Rgba::WHITE);
		strike_pos = _pos;
		break;
	}
}
